package com.sztzjy.forex.trading_trading.controller;


import com.sztzjy.forex.trading_trading.annotation.AnonymousAccess;
import com.sztzjy.forex.trading_trading.annotation.OperateLog;
import com.sztzjy.forex.trading_trading.config.security.JwtUser;
import com.sztzjy.forex.trading_trading.config.security.LoginResult;
import com.sztzjy.forex.trading_trading.config.security.TokenProvider;
import com.sztzjy.forex.trading_trading.util.ResultEntity;
import com.sztzjy.forex.trading_trading.util.RsaUtil;
import com.sztzjy.forex.trading_trading.util.TzApi;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;

@Api(tags = "用户管理模块")
@RestController
@RequestMapping("api/sysUser")
@RequiredArgsConstructor
public class UserController {

    @AnonymousAccess
    @OperateLog(description = "用户登录")
    @ApiOperation(value = "用户登录", httpMethod = "POST")
    @PostMapping("login")
    public ResultEntity<LoginResult> login(@ApiParam("用户名") @RequestParam String username,
                                           @ApiParam("加密后的密文") @RequestParam String passwordEncode) {

        String password;
        try {
            password = RsaUtil.decryptByPrivateKey(passwordEncode);
        } catch (Exception e) {
            throw new IllegalArgumentException("密码错误");
        }
        String md5Pwd =RsaUtil.calculateMD5(password);
        String hashPwd = RsaUtil.formatHash(md5Pwd);
        JwtUser jwtUser = TzApi.foreignExchangeTradingLogin(username, hashPwd);
        Authentication authentication  = new UsernamePasswordAuthenticationToken(jwtUser, null, jwtUser.getAuthorities());
        SecurityContextHolder.getContext().setAuthentication(authentication);
        String token = TokenProvider.createToken(jwtUser);
        return new ResultEntity<LoginResult>(LoginResult.create(jwtUser, token));
    }


    @AnonymousAccess
    @OperateLog(description = "通过智云平台token单点登录")
    @ApiOperation(value = "通过智云平台token单点登录", httpMethod = "POST")
    @PostMapping("loginByZhiYunToken")
    public ResultEntity<LoginResult> loginByZhiYunToken(@ApiParam("智云平台token") @RequestParam String zhiYunToken) {
        Assert.isTrue(StringUtils.hasText(zhiYunToken), "token不能为空");
        JwtUser jwtUser = TokenProvider.getJWTUserByZhiYun(zhiYunToken);
        Authentication authentication  = new UsernamePasswordAuthenticationToken(jwtUser, null, jwtUser.getAuthorities());
        SecurityContextHolder.getContext().setAuthentication(authentication);
        String token = TokenProvider.createToken(jwtUser);
        return new ResultEntity<LoginResult>(LoginResult.create(jwtUser, token));
    }

}



