package com.sztzjy.forex.trading_trading.service;

import com.sztzjy.forex.trading_trading.entity.Log;
import com.sztzjy.forex.trading_trading.repository.ILogRepository;
import com.sztzjy.forex.trading_trading.util.PageUtil;
import lombok.RequiredArgsConstructor;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.persistence.criteria.Predicate;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 系统日志服务层
 */
@Service
@RequiredArgsConstructor
public class LogService {
    private final ILogRepository logRepository;

    public Page<Log> findAll(Integer index, Integer size) {
        PageUtil pageUtil = new PageUtil(index, size);
        pageUtil.addSort("operatedTime", false);
        return logRepository.findAll(pageUtil.getPageable());
    }

    @Transactional(rollbackFor = Exception.class)
    public Log add(Log logRecord) {
        logRecord.setCreateTime(new Date());
        logRepository.save(logRecord);
        return logRecord;
    }

    public Page<Log> findByConditions(Integer index, Integer size, String ipAddress, Long startTime, Long endTime, String operatorName) {
        Specification<Log> specification = (Specification<Log>) (root, criteriaQuery, criteriaBuilder) -> {
            List<Predicate> andPredicate = new ArrayList<>();
            if (ipAddress != null && !ipAddress.isEmpty()) {
                andPredicate.add(criteriaBuilder.equal(root.get("ipAddress"), ipAddress));
            }
            if (startTime != null && startTime != 0) {
                andPredicate.add(criteriaBuilder.greaterThanOrEqualTo(root.get("createTime"), new Date(startTime)));
            }
            if (endTime != null && endTime != 0) {
                andPredicate.add(criteriaBuilder.lessThanOrEqualTo(root.get("createTime"), new Date(endTime)));
            }
            if (operatorName != null && !operatorName.isEmpty()) {
                andPredicate.add(criteriaBuilder.like(root.get("operatorName"), operatorName + '%'));
            }
            return criteriaQuery.where(andPredicate.toArray(new Predicate[]{})).getRestriction();
        };
        PageUtil pageUtil = new PageUtil(index, size);
        pageUtil.addSort("createTime", false);
        return logRepository.findAll(specification, pageUtil.getPageable());
    }
}
